// =================================================================
//
// CBPFBO_Stepper.cpp
// 20-08-21
// Library for Stepper Motor (Harry) Robot
//
// =================================================================
//#define dx 0.07225
//#include <WProgram.h>
#include "CBPFBO_Stepper.h"

#define rob_dia 94

int sspeed;
int speedL;
int speedR;
float prepR;
int doneL,doneR;

void stepMotors(int stepLeft, int stepRight) {
    stepperR.step(stepRight);
    stepperL.step(-stepLeft);
}

int ramp(int vMin, int vMax, int N, float alpha, int n, bool upRamp, bool downRamp) {

  int vely;
  
  if((n < alpha*N) && (upRamp == true)) {
    vely = n*(vMax - vMin)/(alpha*N) + vMin;
  }
  else if ((n < alpha*N) && (upRamp == false)) {
    vely = vMax;
  }
  
  if((n >= alpha*N) && (n < (N - alpha*N))) {
    vely = vMax;
  }

  if((n >= (N - alpha*N)) && (downRamp == true)) {
    vely = (N - n)*(vMax - vMin)/(alpha*N) + vMin;
  }
  else if ((n >= (N - alpha*N)) && (downRamp == false)) {
    vely = vMax;
  }

  if(upRamp == false && downRamp == false) {
    vely = vMax;
  }

  return vely;
  
}

void straightLine(int dist, int vMax, bool upRamp, bool downRamp) {
  int i=0;
  unsigned long steps;

  steps = dist/dx;

  while(i <= steps) {
    sspeed = ramp(1, vMax, steps, 0.25, i, upRamp, downRamp);
    stepperR.setSpeed(sspeed);
    stepperL.setSpeed(sspeed);
    stepMotors(1, 1);
    i++;
  } 
}

int detectCorner(int16_t angs[]) {
  for (int i = 0; i < 3; i++) {
    if ((angs[i] >= 85) && (angs[i] <= 95)) {
      return 90;
    }
    if ((angs[i] >= -95) && (angs[i] <= -85)) {
      return -90;
    } 
  } 
  return 0;
}

// Function for path tracking. Moves forward, detects junction, breaks
// out of movement while(...) then advances half a track width, then
// samples juntion type to return. Then advances until wheels on then
// junction, then returns.
	
int moveForward(int vMax, int dist) {
  int i=0;
  int junctionNumber = 0;
  unsigned long steps;
  long leftSensorVal  = 0;
  long rightSensorVal = 0;
  unsigned int extraSteps;
  bool doneBudge = false;
  int savedSpeed;

  float error;

  int angleDetected;
  
  steps = dist/dx;
  //extraSteps = ceil(9.0/dx);

    Serial.print("steps = ");
    Serial.println(steps);
    
  // principal while loop to move forward
  digitalWrite(24, HIGH);
  while(i <= steps) {
    savedSpeed = vMax;
    sspeed = ramp(1, vMax, steps, 0.25, i, true, false);


    
    stepperR.setSpeed(sspeed);
    stepperL.setSpeed(sspeed);
    stepMotors(10, 10);

    // gain = 0.75*
    // deltaT = 0.80

    // gain = 0.55*
    // deltaT = 0.50

    // gain = 0.25*
    // deltaT = 0.10

    // gain = 0.30*
    // deltaT = 0.35

    //error = 0.25*getError();
   //moveABit(error, 0.10);

    int16_t angs[3];
    for(int i = 0; i < 3; i++) {
       angs[i] = pixy.line.intersections[0].m_intLines[i].m_angle;
       Serial.print("angs[");
       Serial.print(i);
       Serial.print("] = ");
       Serial.println(angs[i]);
    }

    angleDetected = detectCorner(angs);

    if (angleDetected == 90) {
      junctionNumber = LEFT;
      break;
    }
    else if (angleDetected == -90) {
      junctionNumber = RIGHT;
      break;
    }

    i++;
  } 
  digitalWrite(24, LOW); 

    sspeed = savedSpeed;
    stepperR.setSpeed(sspeed);
    stepperL.setSpeed(sspeed); 

     /*
    // move forward to centre of track, sampling track
    int j = 0;
    digitalWrite(26, HIGH);
    while(j < extraSteps) {
      stepMotors(1,1);     
      rightSensorVal = RCTime(17);
      leftSensorVal  = RCTime(14);
      j++;
    }
    digitalWrite(26, LOW);

    // return type of junction sampled
    if(leftSensorVal >= 500) {
      if (rightSensorVal >= 500) {
        junctionNumber = TEE;
      }
      else {
        junctionNumber = RIGHT;
      }
     }

     else if(rightSensorVal >= 500) {
      if (leftSensorVal >= 500) {
        junctionNumber = TEE;
      }
      else {
        junctionNumber = LEFT;
      }
     }
*/
    int j = 0;
    extraSteps = ceil(75.0/dx);
    digitalWrite(28, HIGH);
    while(j < extraSteps) {
      stepMotors(1,1);     
      j++;
    }
    digitalWrite(28, LOW);
      
    return junctionNumber;
  
}

// pivot function handles positive and negative angles (degrees)
void pivot(float degs) {
  float theta = (PI * degs)/180.0;

  int dir;

  int nL,nR;
  int vL,vR;
  float sC;

  if(theta <= 0) {
    dir = -1;
  }
  else {
    dir = 1;
  }
  sC = dir  * rob_dia/2 * theta;
   
  nL = sC/dx;
  nR = nL;



  int i = 0;
  while(i <= nL) {
    sspeed = ramp(1, 18, nL, 0.25, i, false, false);

    stepperR.setSpeed(sspeed);
    stepperL.setSpeed(sspeed);
    
    stepMotors(dir, -dir);
    i++;
  }
}


void arc(float R, float a, float theta, int vMax, bool upRamp, bool downRamp) {
  
  float nRF, nLF;
  int nR,nL;
  float beta;
  float prepR;
  int doneL,doneR;
  int  vL,vR;
  
  nRF = (theta/dx) * (R - a);
  nLF = (theta/dx) * (R + a);

  nR = ceil(nRF);
  nL = ceil(nLF);

  beta = nRF/nLF;
  prepR = 0;
  doneL = 0;
  doneR = 0;

  vL = 18;
  vR = ceil(beta*(float)vL);

  while (doneL < nL) {

    speedL = ramp(1, vL, nL, 0.25, doneL+1, upRamp, downRamp);   
    speedR = ramp(1, vR, nR, 0.25, doneR+1, upRamp, downRamp);
    
    stepperR.setSpeed(speedR);
    stepperL.setSpeed(speedL);
    
    stepMotors(1, 0);
    doneL += 1;
    prepR += beta;
    
    if(prepR > 1.0) {
        stepMotors(0,1);
        doneR += 1;
        prepR -= 1.0;  
    }
  }

  if (prepR > 0.5) {
    stepMotors(0,1);
    doneR += 1.0;
    prepR -= 1.0;
  }

}

// Get error = leftVal - rightVal. Normalize by maxDiff
float getError() {
  int i = 1;
  long leftSensorVal;
  long rightSensorVal;
  float error;

  rightSensorVal = RCTime(15);
  leftSensorVal = RCTime(16);

  rightSensorVal = map(rightSensorVal,232,838,100,1500);
  leftSensorVal = map(leftSensorVal,210,1161,100,1500);
  error = (float)(leftSensorVal - rightSensorVal) / 2600.0;

  return error;
  
}

// Move depending on error
void moveABit(float error, float deltaT) {
  float vLF, vRF;
  float nLF, nRF;
  float beta;
  float prepR, prepL;
  int speedL, speedR;
  int nR,nL;
  int  vL,vR;
  int doneL,doneR;

if(error < -offset) {
  vLF = 18.0;
  vRF = vLF * error;

  nLF = (vLF/dx) * deltaT;
  nRF = nLF * (vRF/vLF);

  vL = ceil(vLF);
  vR = ceil(vRF);

  nL = ceil(nLF);
  nR = ceil(nRF);

  beta = nRF/nLF;

  prepR = 0;
  doneL = 0;
  doneR = 0;

  while (doneL < nL) {

    speedL = ramp(1, vL, nL, 0.25, doneL+1, false, false);   
    speedR = ramp(1, vR, nR, 0.25, doneR+1, false, false);
    
    stepperR.setSpeed(speedR);
    stepperL.setSpeed(speedL);
    
    stepMotors(1, 0);
    doneL += 1;
    prepR += beta;
    
    if(prepR > 1.0) {
        stepMotors(0,1);
        doneR += 1;
        prepR -= 1.0;  
    }
  }

}
else if(error > offset) {
  vRF = 18.0;
  vLF = vRF * error;

  nRF = (vRF/dx) * deltaT;
  nLF = nRF * (vLF/vRF);

  vL = ceil(vLF);
  vR = ceil(vRF);

  nL = ceil(nLF);
  nR = ceil(nRF);

  beta = nLF/nRF;

  prepL = 0;
  doneL = 0;
  doneR = 0;

  while (doneR < nR) {

    speedL = ramp(1, vL, nL, 0.25, doneL+1, false, false);   
    speedR = ramp(1, vR, nR, 0.25, doneR+1, false, false);
    
    stepperR.setSpeed(speedR);
    stepperL.setSpeed(speedL);
    
    stepMotors(0, 1);
    doneR += 1;
    prepL += beta;
    
    if(prepL > 1.0) {
        stepMotors(1,0);
        doneL += 1;
        prepL -= 1.0;  
    }
  }
}

else {
  speedL = 18;
  speedR = 18;

  stepperR.setSpeed(speedR);
  stepperL.setSpeed(speedL);

  stepMotors(5, 5);
}
 
}
