// =================================================================
//
// CBPFBO_StepperA.cpp
// 24-08-21
// Latest Rev. 08-01-23
// Library for Stepper Motor (Harry) Robot
//
// arc(...) can now handle negative angles
// new function setRobotParams() so user can set from sketch
// new function getDX() to return calculated dx in library.
// 18-01-23 int changed to long on steps, nR, nL calculations
//
// =================================================================

#include "CBPFBO_StepperA.h"
#include <Pixy2.h>

// Variables needed by this class ----------------------------------
float delx;
int rob_dia;
int sspeed;
int speedL;
int speedR;
float prepR;
int doneL,doneR;

const int stepsPerRevolution = 2038; 
 
Stepper stepperR(stepsPerRevolution, 4, 6, 5, 7);
Stepper stepperL(stepsPerRevolution, 8, 10, 9, 11);

// public functions -------------------------------------------------
void setRobotParams(float wheelDia_, float axleLen_){
	delx = ((PI*wheelDia_)/(float)stepsPerRev);
	rob_dia = axleLen_;
	
}

float getDX() {
	return delx;
}

void stepMotors(int stepLeft, int stepRight) {
    stepperR.step(stepRight);
    stepperL.step(-stepLeft);
}

void setStepperSpeeds(int leftSpeed, int rightSpeed) {
	stepperR.setSpeed(rightSpeed);
    stepperL.setSpeed(leftSpeed);
}

int ramp(int vMin, int vMax, unsigned long N, float alpha, unsigned long n, bool upRamp, bool downRamp) {

  int vely;
  
  if((n < alpha*N) && (upRamp == true)) {
    vely = n*(vMax - vMin)/(alpha*N) + vMin;
  }
  else if ((n < alpha*N) && (upRamp == false)) {
    vely = vMax;
  }
  
  if((n >= alpha*N) && (n < (N - alpha*N))) {
    vely = vMax;
  }

  if((n >= (N - alpha*N)) && (downRamp == true)) {
    vely = (N - n)*(vMax - vMin)/(alpha*N) + vMin;
  }
  else if ((n >= (N - alpha*N)) && (downRamp == false)) {
    vely = vMax;
  }

  if(upRamp == false && downRamp == false) {
    vely = vMax;
  }

  return vely;
  
}

void straightLine(int dist, int vMax, bool upRamp, bool downRamp) {
  unsigned long i=0;
  unsigned long steps;

  steps = dist/delx;

  while(i <= steps) {
    sspeed = ramp(1, vMax, steps, 0.25, i, upRamp, downRamp);
    stepperR.setSpeed(sspeed);
    stepperL.setSpeed(sspeed);
    stepMotors(1, 1);
    i++;
  } 
}

int detectCorner(int16_t angs[]) {
  for (int i = 0; i < 3; i++) {
    if ((angs[i] >= 85) && (angs[i] <= 95)) {
		return 90;
    }
    if ((angs[i] >= -95) && (angs[i] <= -85)) {
		return -90;		
    } 
  } 
  return 0;
}

int detectCornersOrig(int16_t angs[]) {
	int16_t detAngs[4];
	int output;
  for (int i = 0; i < 4; i++) {
	detAngs[i] = 0;
  }
  for (int i = 0; i < 4; i++) {
    if ((angs[i] >= 85) && (angs[i] <= 95)) {
		detAngs[0] = 90;
    }
    if ((angs[i] >= -95) && (angs[i] <= -85)) {
		detAngs[1] = -90;		
    } 
  }
  output = NONE;
  if (((detAngs[0] == -90) && (detAngs[1] == 90)) || ((detAngs[1] == -90) && (detAngs[0] == 90))) {
	  output = TEE;
  }
  
  if (((detAngs[0] == 0) && (detAngs[1] == 90)) || ((detAngs[1] == 0) && (detAngs[0] == 90))) {
	  output = LEFT;
  }
  
  if (((detAngs[0] == -90) && (detAngs[1] == 0)) || ((detAngs[1] == -90) && (detAngs[0] == 0))) {
	  output = RIGHT;
  }
  return output;
}

int detectCorners(int16_t angs[]) {
	int16_t detAngs[4];
	int output;
  for (int i = 0; i < 4; i++) {
	detAngs[i] = 0;
  }
  for (int i = 0; i < 4; i++) {
    if ((angs[i] >= 75) && (angs[i] <= 105)) {
		detAngs[0] = 90;
    }
    if ((angs[i] >= -105) && (angs[i] <= -75)) {
		detAngs[1] = -90;		
    } 
  }
  output = NONE;
  if (((detAngs[0] == -90) && (detAngs[1] == 90)) || ((detAngs[1] == -90) && (detAngs[0] == 90))) {
	  output = TEE;
  }
  
  if (((detAngs[0] == 0) && (detAngs[1] == 90)) || ((detAngs[1] == 0) && (detAngs[0] == 90))) {
	  output = LEFT;
  }
  
  if (((detAngs[0] == -90) && (detAngs[1] == 0)) || ((detAngs[1] == -90) && (detAngs[0] == 0))) {
	  output = RIGHT;
  }
  return output;
}

// Function for path tracking. Moves forward, detects junction, breaks
// out of movement while(...) then advances half a track width, then
// samples juntion type to return. Then advances until wheels on then
// junction, then returns.
	
int moveForward(Pixy2 pixy, int vMax, int dist) {
  int i=0;
  int junctionNumber = 0;
  unsigned long steps;
  long leftSensorVal  = 0;
  long rightSensorVal = 0;
  unsigned int extraSteps;
  bool doneBudge = false;
  int savedSpeed;

  float error;

  int angleDetected;
  
  steps = dist/delx;
  
  Serial.print("steps = ");
  Serial.println(steps);
    
  // principal while loop to move forward
  digitalWrite(24, HIGH);
  while(i <= steps) {
    savedSpeed = vMax;
    sspeed = ramp(1, vMax, steps, 0.25, i, true, false);

    stepperR.setSpeed(sspeed);
    stepperL.setSpeed(sspeed);
    stepMotors(10, 10);

    int16_t angs[3];
    for(int i = 0; i < 3; i++) {
       angs[i] = pixy.line.intersections[0].m_intLines[i].m_angle;
       Serial.print("angs[");
       Serial.print(i);
       Serial.print("] = ");
       Serial.println(angs[i]);
    }

    angleDetected = detectCorner(angs);

    if (angleDetected == 90) {
      junctionNumber = LEFT;
      break;
    }
    else if (angleDetected == -90) {
      junctionNumber = RIGHT;
      break;
    }

    i++;
  } 
  digitalWrite(24, LOW); 

    sspeed = savedSpeed;
    stepperR.setSpeed(sspeed);
    stepperL.setSpeed(sspeed); 
	
    int j = 0;
    extraSteps = ceil(75.0/delx);
    digitalWrite(28, HIGH);
    while(j < extraSteps) {
      stepMotors(1,1);     
      j++;
    }
    digitalWrite(28, LOW);
      
    return junctionNumber;
  
}

// pivot function handles positive and negative angles (degrees)
void pivot(float degs, int vMax, bool upRamp,bool downRamp) {
  float theta = (PI * degs)/180.0;

  int dir;

  int nL,nR;
  int vL,vR;
  float sC;

  if(theta <= 0) {
    dir = -1;
  }
  else {
    dir = 1;
  }
  sC = dir  * rob_dia/2 * theta;
   
  nL = sC/delx;
  nR = nL;



  int i = 0;
  while(i <= nL) {
    sspeed = ramp(1, vMax, nL, 0.25, i, upRamp, downRamp);

    stepperR.setSpeed(sspeed);
    stepperL.setSpeed(sspeed);
    
    stepMotors(dir, -dir);
    i++;
  }
}


void arc(float R, float degs, int vMax, bool upRamp, bool downRamp) {
  
  float nRF, nLF;
  unsigned long nR,nL;
  float beta;
  float prepR;
  float prepL;
  unsigned long doneL,doneR;
  int  vL,vR;
  float theta;
  float a;
  
  a = rob_dia/2.0;
  theta = degs*PI/180.0;
  
  if(theta > 0.0) {
	nRF = (theta/delx) * (R - a);
	nLF = (theta/delx) * (R + a);

	nR = ceil(nRF);
	nL = ceil(nLF);

	beta = nRF/nLF;
	prepR = 0;
	doneL = 0;
	doneR = 0;

	vL = 18;
	vR = ceil(beta*(float)vL);

	while (doneL < nL) {

		speedL = ramp(1, vL, nL, 0.25, doneL+1, upRamp, downRamp);   
		speedR = ramp(1, vR, nR, 0.25, doneR+1, upRamp, downRamp);
    
		stepperR.setSpeed(speedR);
		stepperL.setSpeed(speedL);
    
		stepMotors(1, 0);
		doneL += 1;
		prepR += beta;
    
		if(prepR > 1.0) {
			stepMotors(0,1);
			doneR += 1;
			prepR -= 1.0;  
		}
	}

	if (prepR > 0.5) {
		stepMotors(0,1);
		doneR += 1.0;
		prepR -= 1.0;
	}
 
	  
  } else {
	theta = -theta;
	nLF = (theta/delx) * (R - a);
	nRF = (theta/delx) * (R + a);

	nR = ceil(nRF);
	nL = ceil(nLF);

	beta = nLF/nRF;
	prepL = 0;
	doneL = 0;
	doneR = 0;

	vR = 18;
	vL = ceil(beta*(float)vR);

	while (doneR < nR) {

		speedL = ramp(1, vL, nL, 0.25, doneL+1, upRamp, downRamp);   
		speedR = ramp(1, vR, nR, 0.25, doneR+1, upRamp, downRamp);
    
		stepperR.setSpeed(speedR);
		stepperL.setSpeed(speedL);
    
		stepMotors(0, 1);
		doneR += 1;
		prepL += beta;
    
		if(prepL > 1.0) {
			stepMotors(1,0);
			doneL += 1;
			prepL -= 1.0;  
		}
	}

	if (prepL > 0.5) {
		stepMotors(1,0);
		doneL += 1.0;
		prepL -= 1.0;
	}
  
	  
  }
 
}

// Get error = leftVal - rightVal. Normalize by maxDiff

float getError() {
  int i = 1;
  long leftSensorVal;
  long rightSensorVal;
  float error;

  rightSensorVal = RCTime(15);
  leftSensorVal = RCTime(16);

  rightSensorVal = map(rightSensorVal,232,838,100,1500);
  leftSensorVal = map(leftSensorVal,210,1161,100,1500);
  error = (float)(leftSensorVal - rightSensorVal) / 2600.0;

  return error;
  
}

// From experiments with Parallax Line sensors
long RCTime(int sensorIn){
   long duration = 0;
   pinMode(sensorIn, OUTPUT);     // Make pin OUTPUT
   digitalWrite(sensorIn, HIGH);  // Pin HIGH (discharge capacitor)
   delay(1);                      // Wait 1ms
   pinMode(sensorIn, INPUT);      // Make pin INPUT
   digitalWrite(sensorIn, LOW);   // Turn off internal pullups
   while(digitalRead(sensorIn)){  // Wait for pin to go LOW
      duration++;
   }
   return duration;
}

// Move depending on error
void moveABit(float error, float deltaT) {
  float vLF, vRF;
  float nLF, nRF;
  float beta;
  float prepR, prepL;
  int speedL, speedR;
  int nR,nL;
  int  vL,vR;
  int doneL,doneR;

if(error < 0) {
  vLF = 18.0;
  vRF = vLF * (1 + error);

  nLF = (vLF/delx) * deltaT;
  nRF = nLF * (vRF/vLF);

  vL = ceil(vLF);
  vR = ceil(vRF);

  nL = ceil(nLF);
  nR = ceil(nRF);

  beta = nRF/nLF;

  prepR = 0;
  doneL = 0;
  doneR = 0;

  while (doneL < nL) {

    speedL = ramp(1, vL, nL, 0.25, doneL+1, false, false);   
    speedR = ramp(1, vR, nR, 0.25, doneR+1, false, false);
    
    stepperR.setSpeed(speedR);
    stepperL.setSpeed(speedL);
    
    stepMotors(1, 0);
    doneL += 1;
    prepR += beta;
    
    if(prepR > 1.0) {
        stepMotors(0,1);
        doneR += 1;
        prepR -= 1.0;  
    }
  }
  
  doneR += 1;

}
else if(error > 0) {
  vRF = 18.0;
  vLF = vRF * (1 - error);

  nRF = (vRF/delx) * deltaT;
  nLF = nRF * (vLF/vRF);

  vL = ceil(vLF);
  vR = ceil(vRF);

  nL = ceil(nLF);
  nR = ceil(nRF);

  beta = nLF/nRF;

  prepL = 0;
  doneL = 0;
  doneR = 0;

  while (doneR < nR) {

    speedL = ramp(1, vL, nL, 0.25, doneL+1, false, false);   
    speedR = ramp(1, vR, nR, 0.25, doneR+1, false, false);
    
    stepperR.setSpeed(speedR);
    stepperL.setSpeed(speedL);
    
    stepMotors(0, 1);
    doneR += 1;
    prepL += beta;
    
    if(prepL > 1.0) {
        stepMotors(1,0);
        doneL += 1;
        prepL -= 1.0;  
    }
  }
  doneL += 1;
}

else {
  speedL = 18;
  speedR = 18;

  stepperR.setSpeed(speedR);
  stepperL.setSpeed(speedL);

  stepMotors(5, 5);
}
 
}
